#ifndef __GRAFO__
#define __GRAFO__

#define INFINITY 999
#define MAXNODES NUMERO_DE_NOS
#define MEMBER 1
#define NONMEMBER 0

#include "fila.h"
#include <stdio.h>


void imprimir_grafo(int grafo[][NUMERO_DE_NOS])
{
    int i, j,k=1,l=0;
	/* Cria uma janela usando as bibliotecas do ncurses */
	WINDOW *janela_imprimir;
	PANEL *painel_imprimir;
	janela_imprimir=newwin(10,40,6,20);
	wbkgd(janela_imprimir,COLOR_PAIR(1));
	painel_imprimir=new_panel(janela_imprimir);
	for(i=0; i<NUMERO_DE_NOS; i++)
	{
		wprintw(janela_imprimir,"\n");
		for(j=0; j<NUMERO_DE_NOS; j++)
			wprintw(janela_imprimir," %.4d ", grafo[i][j]);	
	}
	wprintw(janela_imprimir,"\n\n");
	wattron(janela_imprimir,COLOR_PAIR(4));
	wprintw(janela_imprimir,"Pressione qualquer tecla para continuar");
	wattroff(janela_imprimir,COLOR_PAIR(4));
	show_panel(painel_imprimir);
    update_panels();
    doupdate();
    getch();
}

void iniciar_grafo(int grafo[][NUMERO_DE_NOS])
{
	int i, j;
	
	for(i=0; i<NUMERO_DE_NOS; i++)
		for(j=0; j<NUMERO_DE_NOS; j++)
			grafo[i][j] = INFINITY;
}

int ler_grafo(int grafo[][NUMERO_DE_NOS])
{
	
	/* vetor com o valor do caminho A->B, caminho ta na matriz nos */
	int valor[NUMERO_DE_NOS], i, distancia;
	
	/* Grafo de caminhos A -> B, indexado com o vetor valor */
	char nos[NUMERO_DE_NOS][2], de, para;
	
	/* Matriz contendo se existe caminho de um no A para o B, 1 se existir, 0 se nao */
	//int grafo[NUMERO_DE_NOS][NUMERO_DE_NOS];
	
	FILE *arquivo;
	
	arquivo = fopen("grafo.txt", "rt");
	
	if(arquivo == NULL)
	{
		printf("\n Error ao abrir o arquivo(nao existe, ou corrompido.");
		return(-1);
	}
	
	iniciar_grafo(grafo);
	
	for(i=0; !feof(arquivo); i++)
	{
		
		fscanf(arquivo, "%c %c %d", &de, &para, &distancia);
		getc(arquivo);
			
		nos[i][0] = de;
		nos[i][1] = para;
		valor[i] = distancia;
		
		/* Grafo de caminhos, se existe caminho de A para B, recebe 1
		 * menos 97 porque eh o valor da letra A no alfabeto
		 */
		grafo[de-97][para-97] = distancia;
	}
	
	fclose(arquivo);
}

void dijkstra (int g[][MAXNODES], int s, int t)
{
	WINDOW *janela_dijkstra;
	PANEL *painel_dijkstra;
	janela_dijkstra=newwin(10,40,9,20);
	wbkgd(janela_dijkstra,COLOR_PAIR(1));
	painel_dijkstra=new_panel(janela_dijkstra);

    int dist[MAXNODES], perm[MAXNODES], path[MAXNODES];
    int current, i, k, dc;
    int smalldist, newdist;

    /* Inicializa todos os índices de 'perm' como 0 e de 'dist' como INFINITY */
    for (i = 0; i < MAXNODES; i++) {
        perm[i] = NONMEMBER;
        dist[i] = INFINITY;
    }

    /* Inclui 's' em perm (perm[s]=1) e configura(armazena) sua distancia como sendo zero */
    perm[s] = MEMBER;
    dist[s] = 0;

    /* define 's' como origem (fonte) da busca */
    current = s;
    k = current;

    while (current != t) {

        smalldist = INFINITY;
        dc = dist[current];

        for (i = 0; i < MAXNODES; i++) {

            //se o elemento NÃO está em perm
            if (perm[i] == NONMEMBER) {

                //calcula distância a partir do vértice corrente ao vértice adjacente i
                newdist = dc + g[current][i];

                //se a distância partindo do vértice corrente for menor, atualiza o vetor 
                //de distâncias e de precedência
                if (newdist < dist[i]) {
                    dist[i] = newdist;
                    path[i] = current;
                }

                //determina o vértice (entre todos os não pertencentes a perm) com menor distância
                if (dist[i] < smalldist) {
                    smalldist = dist[i];
                    k = i;
                }

            } 

        } /* fim for */

        /* embora estamos assumindo grafos ponderados e conexos, este if garante que
         em caso de não existência de um caminho o programa não entre em loop infinito */
        if (current == k) {
			wprintw(janela_dijkstra,"\n\nCAMINHO NAO EXISTE\n\n");	
            return;
        }

        current = k;
        perm[current] = MEMBER;

    } /* fim while */

    /* impressao do resultado ****************/

    wprintw(janela_dijkstra,"\n\nResultado\n\n");	
	int caminho = t;

    wprintw(janela_dijkstra,"%d <- ", t);	

    while (caminho != s)
    {
        wprintw(janela_dijkstra,"%d", path[caminho]);	
        caminho = path[caminho];

        if (caminho != s)
        wprintw(janela_dijkstra," <- ");	
    }

    wprintw(janela_dijkstra,"\n\ncusto: %d\n\n", dist[t]);	
    /****************************************/
	show_panel(painel_dijkstra);
    update_panels();
    doupdate();
    getch();

} /* fim dijkstra */

#define N NUMERO_DE_NOS

int nextsucc(int adj[][NUMERO_DE_NOS], int xptr, int yptr)
{
	yptr++;
	
	while(adj[xptr][yptr] == 0 && yptr < N)
		yptr++;
	
	return(yptr);
}


void visit(int x)
{
	casa[last++] = x;
}

int visited(int x)
{
	int i;
	for(i=0; i<last; i++)
		if(casa[i]==x)
			return 1;
		
	return 0;
}

int firstsucc(int adj[][NUMERO_DE_NOS], int xptr)
{
	int yptr;
	
	yptr = nextsucc(adj, xptr, -1);
	
	return(yptr);
}


void dftraverse(int adj[][NUMERO_DE_NOS], int xptr)
{
	WINDOW *janela_dftraverse;
	PANEL *painel_dftraverse;
	janela_dftraverse=newwin(10,40,6,20);
	wbkgd(janela_dftraverse,COLOR_PAIR(1));
	painel_dftraverse=new_panel(janela_dftraverse);

	int yptr;
			
	visit(xptr);	/* inseri xptr na lista de nos visitados */
	yptr = firstsucc(adj, xptr);
	
	while(yptr < N)
	{
		if(!visited(yptr))
			dftraverse(adj, yptr);
		wprintw(janela_dftraverse,"\n%d", adj[yptr][xptr]);	
//		fprintf(stderr,"\n%d", adj[yptr][xptr]);		
		yptr = nextsucc(adj, xptr, yptr);
	
	}
	show_panel(painel_dftraverse);
    update_panels();
    doupdate();
}



void bftraverse(int adj[][NUMERO_DE_NOS], int xptr)
{
	int yptr;
	
	Queue queues;
	queues = CreateQueue(NUMERO_DE_NOS);
	
	visit(xptr);
	
	Enqueue(yptr, queues);
	
	while(!IsEmpty(queues))
	{
		xptr = FrontAndDequeue(queues);
		yptr = firstsucc(adj, xptr);
		
		while(yptr < N)
		{
		
			if( !visited(yptr) )
			{
				visit(yptr);
				Enqueue(yptr, queues);
			}
		}
		
		yptr = nextsucc(adj, xptr, yptr);
	}
}

#endif
