/***************************************************************************
 *            grafos_ncurses.c
 *
 *  Tue Mai 05 10:53:55 2004
 *  Copyright          Renan Mathias Fernandes RA: 02513356
 *							email:	renanmathias at terra.com.br
 *  						Cesar Henrique Kallas		RA: 02099224
 *							email:	opensrc at gmx.net 
 *
 * ATENCAO:
 * Para compilar esse programa, voce deve usar o gcc e especificar
 * que esta utilizando o libncurses (Plataforma gráfica para console)
 * Sintaxe para compilar :
 * gcc grafos_ncurses.c -o grafos_ncurses -lncurses -lpanel
 ****************************************************************************/

#define ENTER 10
#define ESCAPE 27

#define NUMERO_DE_NOS 5
int casa[NUMERO_DE_NOS];
int last=0;

#include <panel.h>
#include <stdlib.h>
#include "grafo.h"
#include "menu.h"

void desenha_menubar(WINDOW *);
WINDOW **desenha_menu(int);
void delete_menu(WINDOW **,int);
int scroll_menu(WINDOW **,int,int);
void init_curses(void);
void janela_informacoes(WINDOW *,PANEL *);
int dados_um();

int dados_um()
{
	echo();
	int valor;
	WINDOW *janela_dados;
	PANEL *painel_dados;
	janela_dados=newwin(3,30,5,6);
	wbkgd(janela_dados,COLOR_PAIR(1));
	painel_dados=new_panel(janela_dados);
	wborder(janela_dados,0,0,0,0,0,0,0,0);		
	
	wattron(janela_dados,COLOR_PAIR(4));
    mvwprintw(janela_dados,1,1,"Entre com o valor:");
    wattroff(janela_dados,COLOR_PAIR(4));
	show_panel(painel_dados);
    update_panels();
    doupdate();

	wattron(janela_dados,COLOR_PAIR(4));
    move(6,26);
	scanw("%d",&valor);
    wattroff(janela_dados,COLOR_PAIR(4));
	return valor;
}

void limpa()
{
	move(20,1);
	printw("                                                ");
}

void desenha_menubar(WINDOW *menubar)
{
	wbkgd(menubar,COLOR_PAIR(1));
	waddstr(menubar,"Operacoes com Grafos ");
	wattron(menubar,COLOR_PAIR(4));
	waddstr(menubar,"(F2)");
}


WINDOW **desenha_menu(int start_col)
{
	int i;
	WINDOW **items;
	items=(WINDOW **)malloc(9*sizeof(WINDOW *));
    items[0]=newwin(10,19,1,start_col);
	wbkgd(items[0],COLOR_PAIR(1));
	box(items[0],ACS_VLINE,ACS_HLINE);
	items[1]=subwin(items[0],1,17,2,start_col+1);
	items[2]=subwin(items[0],1,17,3,start_col+1);
	items[3]=subwin(items[0],1,17,4,start_col+1);
	items[4]=subwin(items[0],1,17,5,start_col+1);
	items[5]=subwin(items[0],1,17,6,start_col+1);
	wprintw(items[1],"Abre o grafo");
	wprintw(items[2],"Imprime o grafo");
	wprintw(items[3],"Dijkstra");
	wprintw(items[4],"Dftraverse");
    wprintw(items[5],"Bftraverse");	
	wbkgd(items[1],COLOR_PAIR(2));
	wrefresh(items[0]);
	return items;
}


void delete_menu(WINDOW **items,int count)
{
	int i;
	for (i=0;i<count;i++)
		delwin(items[i]);
	free(items);
}

int scroll_menu(WINDOW **items,int count,int menu_start_col)
{
	int key;
	int selected=0;
	while (1) {
		key=getch();
		if (key==KEY_DOWN || key==KEY_UP) {
			wbkgd(items[selected+1],COLOR_PAIR(1));
			wnoutrefresh(items[selected+1]);
			if (key==KEY_DOWN) {
				selected=(selected+1) % count;
			} else {
				selected=(selected+count-1) % count;
			}
			wbkgd(items[selected+1],COLOR_PAIR(2));
			wnoutrefresh(items[selected+1]);
			doupdate();
		} else if (key==KEY_LEFT || key==KEY_RIGHT) {
			delete_menu(items,count+1);
			touchwin(stdscr);
			refresh();
			items=desenha_menu(20-menu_start_col);
			return scroll_menu(items,8,20-menu_start_col);
		} else if (key==ESCAPE) {
			return -1;
		} else if (key==ENTER) {
			return selected;
		}
	}
}
void init_curses()
{
	initscr();
	start_color();
    init_pair(1, COLOR_BLACK, COLOR_WHITE);	
	init_pair(2, COLOR_WHITE, COLOR_BLUE);
	init_pair(3, COLOR_WHITE, COLOR_RED);
    init_pair(4, COLOR_RED, COLOR_WHITE);
	curs_set(0);
	cbreak();
	noecho();
}

void janela_informacoes(WINDOW *janela, PANEL *painel)
{
	int lines = 8, cols = 40, y = 3, x = 4;
    janela = newwin(lines, cols, y, x);
	wbkgd(janela,COLOR_PAIR(1));
  
	/* Cria borda em volta do painel   */
	box(janela, 0, 0);
	wattron(janela,COLOR_PAIR(4));
    mvwprintw(janela,1,1,"Integrantes do grupo:");
    wattroff(janela,COLOR_PAIR(4));
	mvwprintw(janela,3,1,"Renan Mathias Fernandes RA: 02513356");
	mvwprintw(janela,4,1,"Cesar Henrique Kallas	RA: 02099224");
	/* Gruda um painel numa janela */
	painel = new_panel(janela);
	/* Atualiza os paineis  */
	update_panels();
	keypad(stdscr,TRUE);
}

int main()
{
	int grafo[NUMERO_DE_NOS][NUMERO_DE_NOS];
	
	int key;
	WINDOW *janela_inf, *menubar, *janela_teste;
	PANEL *painel_informacoes, *painel_teste;
	init_curses();
    bkgd(COLOR_PAIR(2));
	menubar=subwin(stdscr,1,80,0,0);
	desenha_menubar(menubar);
	/* Mostra na tela */
    doupdate();
    janela_informacoes(janela_inf, painel_informacoes);	
	refresh();
	do
	{
		int item_selecionado;
		WINDOW **menu_items;
		key=getch();
		if (key==KEY_F(2)) 
			{
			    menu_items=desenha_menu(0);
			    item_selecionado=scroll_menu(menu_items,8,20);
			    delete_menu(menu_items,9);
			if (item_selecionado<0)
			{
				move(7,1);
				printw("Voce nao selecionou nenhum item");
			}
			else
			{
				switch(item_selecionado)
				{
					case 0:
					{
						move(20,1);
						if (ler_grafo(grafo)==-1)
								printw("Erro ao ler o arquivo!");
							else
								printw("Arquivo Lido!");
							break;
					}
					case 1:
					{
							imprimir_grafo(grafo);
							break;
					}
					case 2:
					{
						int val,val2;
						val = dados_um();
						move(20,1);
						printw("Pedindo agora o segundo valor");
						val2 = dados_um();
						dijkstra(grafo,val,val2);
						limpa();
						break;
					}
					case 3:
					{
    					int val;
						val = dados_um();
						dftraverse(grafo, 0);
						getch();
						break;
					}
					case 4:
					{
						int val;
						val = dados_um();
						last = 0;
						bftraverse(grafo, val);
						break;
					}
						
				}
			}
			touchwin(stdscr);
			refresh();
		}
	} while (key!=ESCAPE);
	endwin();
}
